/*********************************************************************
*                    SEGGER Microcontroller GmbH                     *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*            (c) 2014 - 2020 SEGGER Microcontroller GmbH             *
*                                                                    *
*           www.segger.com     Support: support@segger.com           *
*                                                                    *
**********************************************************************
*                                                                    *
* All rights reserved.                                               *
*                                                                    *
* Redistribution and use in source and binary forms, with or         *
* without modification, are permitted provided that the following    *
* conditions are met:                                                *
*                                                                    *
* - Redistributions of source code must retain the above copyright   *
*   notice, this list of conditions and the following disclaimer.    *
*                                                                    *
* - Neither the name of SEGGER Microcontroller GmbH                  *
*   nor the names of its contributors may be used to endorse or      *
*   promote products derived from this software without specific     *
*   prior written permission.                                        *
*                                                                    *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND             *
* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,        *
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF           *
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE           *
* DISCLAIMED.                                                        *
* IN NO EVENT SHALL SEGGER Microcontroller GmbH BE LIABLE FOR        *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR           *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  *
* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;    *
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF      *
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE  *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH   *
* DAMAGE.                                                            *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : main.c
Purpose : Blinky application for the MIMXRT685-EVK evaluation board

*/

#include <stdio.h>
#include <stdlib.h>

/*********************************************************************
*
*       Defines (fixed)
*
**********************************************************************
*/
#ifndef U8
  #define U8  unsigned char
#endif
#ifndef U16
  #define U16 unsigned short
#endif
#ifndef U32
  #define U32 unsigned long
#endif

#define _CLKCTL1_BASE_ADDR     (0x40021000)
#define _RSTCTL1_BASE_ADDR     (0x40020000)
#define _IOCON_BASE_ADDR       (0x40004000)
#define _GPIO_BASE_ADDR        (0x40100000)

#define _PRSTCTL1_CLR          *(volatile U32*)(_RSTCTL1_BASE_ADDR + 0x0074)

#define _CLKCTL1_PSCCTL1_SET   *(volatile U32*)(_CLKCTL1_BASE_ADDR + 0x0044)

#define _GPIO_DIR0             *(volatile U32*)(_GPIO_BASE_ADDR    + 0x2000)
#define _GPIO_SET0             *(volatile U32*)(_GPIO_BASE_ADDR    + 0x2200)
#define _GPIO_CLR0             *(volatile U32*)(_GPIO_BASE_ADDR    + 0x2280)
#define _GPIO_NOT0             *(volatile U32*)(_GPIO_BASE_ADDR    + 0x2300)

#define _PIO0_14               *(volatile U32*)(_IOCON_BASE_ADDR   + 0x0038)
#define _PIO0_26               *(volatile U32*)(_IOCON_BASE_ADDR   + 0x0068)
#define _PIO0_31               *(volatile U32*)(_IOCON_BASE_ADDR   + 0x007c)

/*********************************************************************
*
*       main()
*
*  Function description
*    The application toggles 3 leds on the MIMXRT685-EVK evaluation board.
*    For details information regarding this, please refer to:
*    https://wiki.segger.com/i.MXRT600
*/
void main(void) {
  int Cnt;
  //
  // Enable and release GPIO0 clock
  //
  _CLKCTL1_PSCCTL1_SET = (1uL << 0);  // Enable clocks for PIO port 0
  _PRSTCTL1_CLR = (1uL << 0);         // Clear reset of GPIO0
  _PIO0_14 = 0;                       // GPIO function
  _PIO0_26 = 0;                       // GPIO function
  _PIO0_31 = 0;                       // GPIO function
  _GPIO_CLR0 = (1uL << 14) | (1uL << 26) | (1uL << 31);   // Turn all LEDs off by default
  _GPIO_DIR0 |= (1uL << 14) | (1uL << 26) | (1uL << 31);  // DIR: Output
  do {
    //
    // Toggle LEDs
    // 
    _GPIO_NOT0 = (1uL << 26) | (1uL << 31) | (1uL << 14);
    for (int i = 0; i < 2000000; i++) {
      __asm("NOP");
    }
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
    Cnt++;
  } while (1);
}

/*************************** End of file ****************************/
